#! /usr/bin/env python3
###############################################################################
#                                                                             #
# GALORE: Gaussian and Lorentzian broadening for simulated spectra            #
#                                                                             #
# Developed by Adam J. Jackson and Alex Ganose                                #
# at University College London (2018)                                         #
#                                                                             #
###############################################################################
#                                                                             #
# This file is part of Galore. Galore is free software: you can redistribute  #
# it and/or modify it under the terms of the GNU General Public License as    #
# published by the Free Software Foundation, either version 3 of the License, #
# or (at your option) any later version.  This program is distributed in the  #
# hope that it will be useful, but WITHOUT ANY WARRANTY; without even the     #
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    #
# See the GNU General Public License for more details.  You should have       #
# received a copy of the GNU General Public License along with this program.  #
# If not, see <http://www.gnu.org/licenses/>.                                 #
#                                                                             #
###############################################################################
from argparse import ArgumentParser
from itertools import cycle

import logging
import numpy as np
from matplotlib import pyplot as plt

from galore.cli import get_default_style
from galore.cross_sections import get_cross_sections_scofield


def main():
    parser = get_parser()
    args = parser.parse_args()
    args = vars(args)
    run(**args)


def get_parser():
    parser = ArgumentParser()
    parser.add_argument('--emin', type=float, default=1,
                        help="Minimum energy in keV")
    parser.add_argument('--emax', type=float, default=20,
                        help="Maximum energy in keV")
    parser.add_argument('--megabarn', action='store_true',
                        help="Set y-axis unit to megabarn/electron")
    parser.add_argument('--size', type=float, nargs=2, default=None,
                        help="Figure dimensions in cm")
    parser.add_argument('--output', '-o', type=str, default=None,
                        help="Output filename. If not given, plot to screen.")
    parser.add_argument('--fontsize', type=int, default=12,
                        help="Font size in pt")
    parser.add_argument(
        '--style', type=str, nargs='+', default=get_default_style(),
        help='Plotting style: a sequence of matplotlib styles and paths to '
             'style files.'
        )
    parser.add_argument('elements', type=str, nargs='+', help="""
        Space-separated symbols for elements in material.""")

    return parser


def run(elements, emin=1, emax=10, megabarn=False, size=None, output=None,
        fontsize=10, style=None):
    energies = np.linspace(emin, emax, 200)
    cross_sections = get_cross_sections_scofield(energies, elements)

    plt.style.use(style)

    if size is None:
        fig = plt.figure()
    else:
        size_inches = [float(x) / 2.54 for x in size]
        fig = plt.figure(figsize=size_inches)

    ax = fig.add_subplot(1, 1, 1)

    colors = cycle(('C0', 'C1', 'C2', 'C3', 'C4'))
    markers = cycle(('o', '^', 'D', 'x', '*'))

    if megabarn:
        unit = "Mb"
        conversion = 1E6
    else:
        unit = "barn"
        conversion = 1

    for element in elements:
        color, marker = next(colors), next(markers)
        linestyles = cycle((['-', '--', ':', '-.']))

        if 'warning' in cross_sections[element]:
            logging.warning("  {0}: {1}".format(
                element, cross_sections[element]['warning']))

        for orbital in 'spdf':
            if (orbital in cross_sections[element] and
                    cross_sections[element][orbital] is not None):
                ax.plot(energies,
                        cross_sections[element][orbital] / conversion,
                        color=color, linestyle=next(linestyles),
                        marker=marker, markevery=40,
                        label='{0}-{1}'.format(element, orbital),
                        markersize=5)

    ax.tick_params(labelsize=(fontsize - 2))
    ax.set_xlabel('Photon energy / keV', fontsize=fontsize)
    ylabel = ('Photoionization cross-section /'
              ' {} electron$^{{-1}}$'.format(unit))
    ax.set_ylabel(ylabel, fontsize=fontsize)
    ax.set_yscale('log')
    ax.legend(loc='lower right', fontsize=fontsize)
    fig.subplots_adjust(right=0.98, top=0.98)

    if output is None:
        plt.show()
    else:
        fig.savefig(output, bbox_inches='tight')


if __name__ == '__main__':
    main()



#! /usr/bin/env python3
###############################################################################
#                                                                             #
# GALORE: Gaussian and Lorentzian broadening for simulated spectra            #
#                                                                             #
# Developed by Adam J. Jackson and Alex Ganose                                #
# at University College London (2018)                                         #
#                                                                             #
###############################################################################
#                                                                             #
# This file is part of Galore. Galore is free software: you can redistribute  #
# it and/or modify it under the terms of the GNU General Public License as    #
# published by the Free Software Foundation, either version 3 of the License, #
# or (at your option) any later version.  This program is distributed in the  #
# hope that it will be useful, but WITHOUT ANY WARRANTY; without even the     #
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    #
# See the GNU General Public License for more details.  You should have       #
# received a copy of the GNU General Public License along with this program.  #
# If not, see <http://www.gnu.org/licenses/>.                                 #
#                                                                             #
###############################################################################
from argparse import ArgumentParser
import galore.cross_sections

def main():
    parser = get_parser()
    args = parser.parse_args()
    args = vars(args)
    run(**args)

def get_parser():
    parser = ArgumentParser()
    parser.add_argument('energy', type=str,
                        help="""
        Photon energy, expressed as source type: "he2" for He (II), "alka" for
        Al k-alpha, (values from Yeh/Lindau (1985)) or as energy in keV (values
        from polynomial fit to Scofield (1973)).""")
    parser.add_argument('elements', type=str, nargs='+', help="""
        Space-separated symbols for elements in material.""")

    return parser

def run(energy, elements):
    cross_sections = galore.get_cross_sections(energy, elements)
    logging = galore.cross_sections.cross_sections_info(cross_sections)
    logging.info("Photoionisation cross sections per electron:")

    for element in elements:
        if 'warning' in cross_sections[element]:
            logging.warning("  {0}: {1}".format(
                element, cross_sections[element]['warning']))
        else:
            orbitals = cross_sections[element]

            for orbital, value in orbitals.items():
                if orbital == 'energy':
                    pass
                if value is None:
                    pass
                else:
                    logging.info("   {0} {1}: {2:.3e}".format(element,
                                                              orbital, value))



from pkg_resources import resource_filename
import numpy as np
import galore
import unittest
from numpy.testing import assert_array_almost_equal


class test_process_pdos(unittest.TestCase):
    def test_lorentzian(self):
        self.assertAlmostEqual(galore.lorentzian(3., f0=1, fwhm=(3 * 2.35482)),
                               0.068238617255)

    def test_broaden(self):
        broadened_data = galore.xy_to_1d(np.array([[1., 0.5], [3., 1.5]]),
                                         range(6),
                                         spikes=False)
        lorentzian = 0.2

        assert_array_almost_equal(
            galore.broaden(broadened_data,
                           d=2,
                           dist='lorentzian',
                           width=lorentzian),
            np.array([
                0.00595715, 1.60246962, 3.19897467, 4.7825862, 0.01190685, 0.
            ]))

    def test_process_pdos(self):
        vasprun = resource_filename(__name__, 'SnO2/vasprun.xml.gz')
        xmin, xmax = (-10, 4)
        weighting = 'Alka'

        plotting_data = galore.process_pdos(input=[vasprun],
                                            gaussian=0.3,
                                            lorentzian=0.2,
                                            xmin=xmin,
                                            xmax=xmax,
                                            weighting=weighting)
        self.assertEqual(plotting_data['O']['energy'][0], -10.0)


if __name__ == '__main__':
    unittest.main()
